@@ -6,7 +6,7 @@ import org.jmx4perl.config.DebugStore;
 import org.jmx4perl.config.Restrictor;
 import org.jmx4perl.config.RestrictorFactory;
 import org.jmx4perl.converter.StringToObjectConverter;
-import org.jmx4perl.converter.attribute.ObjectToJsonConverter;
+import org.jmx4perl.converter.json.ObjectToJsonConverter;
 import org.jmx4perl.handler.*;
 import org.jmx4perl.history.HistoryStore;
 import org.json.simple.JSONObject;
@@ -62,7 +62,7 @@ import java.util.Map;
  * like strings or numbers, collections, arrays and maps are also supported (which
  * translate into the corresponding JSON structure). Additional the OpenMBean types
  * {@link javax.management.openmbean.CompositeData} and {@link javax.management.openmbean.TabularData}
- * are supported as well. Refer to {@link org.jmx4perl.converter.attribute.ObjectToJsonConverter}
+ * are supported as well. Refer to {@link org.jmx4perl.converter.json.ObjectToJsonConverter}
  * for additional information.
  *
  * For the client part, please read the documentation of
@@ -305,6 +305,8 @@ public class AgentServlet extends HttpServlet {
             } catch (MBeanRegistrationException e) {
                 log("Cannot unregister MBean: " + e,e);
             }
+        } else {
+            log("Internal Problem: No ConfigMBean name !");
         }
     }
 
@@ -312,17 +314,21 @@ public class AgentServlet extends HttpServlet {
     @Override
     public void log(String msg) {
         super.log(msg);
-        debugStore.log(msg);
+        if (debugStore != null) {
+            debugStore.log(msg);
+        }
     }
 
     @Override
     public void log(String message, Throwable t) {
         super.log(message,t);
-        debugStore.log(message, t);
+        if (debugStore != null) {
+            debugStore.log(message, t);
+        }
     }
 
     private boolean isDebug() {
-        return debugStore.isDebug();
+        return debugStore != null ? debugStore.isDebug() : false;
     }
 
 }
