@@ -1,6 +1,7 @@
 package org.jmx4perl.handler;
 
 import org.jmx4perl.JmxRequest;
+import org.jmx4perl.config.Restrictor;
 
 import javax.management.*;
 
@@ -33,12 +34,23 @@ import javax.management.*;
  */
 public class ReadHandler extends RequestHandler {
 
+    public ReadHandler(Restrictor pRestrictor) {
+        super(pRestrictor);
+    }
+
+    @Override
     public JmxRequest.Type getType() {
         return JmxRequest.Type.READ;
     }
 
-    public Object handleRequest(MBeanServer server, JmxRequest request)
+    @Override
+    public Object doHandleRequest(MBeanServer server, JmxRequest request)
             throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException {
+        if (!restrictor.isAttributeReadAllowed(request.getObjectName(),request.getAttributeName())) {
+            throw new SecurityException("Reading attribute " + request.getAttributeName() +
+                    " is forbidden for MBean " + request.getObjectNameAsString());
+        }
+
         return server.getAttribute(request.getObjectName(), request.getAttributeName());
     }
 }
