@@ -173,7 +173,6 @@ sub autodetect {
             eval "\$val = \$self->$what";
             die $@ if $@;
         }
-        #print "V: $val";
         return 1 if ($val && (!$pattern || ref($pattern) ne "Regexp"));
         return $val =~ $pattern if ($val && $pattern);
     }
@@ -252,7 +251,14 @@ sub alias {
         $alias = JMX::Jmx4Perl::Alias->by_name($alias_or_name) 
           || croak "No alias $alias_or_name known";
     }
-    my $aliasref = $self->resolve_alias($alias) || $alias->default();
+    my $resolved_ref = $self->resolve_alias($alias);
+    # It has been defined by the handler, but set to 0. So it doesn't 
+    # support this particular alias
+    return undef if (defined($resolved_ref) && !$resolved_ref);
+    # If the handler doesn't define the ref (so it's undef),
+    # use the default
+    my $aliasref =  $resolved_ref || $alias->default();
+    # If there is no default, then there is no support, too.
     return undef unless defined($aliasref);
 
     return $aliasref if (ref($aliasref) eq "CODE"); # return coderefs directly
@@ -296,7 +302,7 @@ sub resolve_alias {
 
 =item my $aliases = $self->init_aliases()
 
-Metho used during construction of a handler for obtaining a translation map of
+Method used during construction of a handler for obtaining a translation map of
 aliases to the real values. Each specific handler can overwrite this method to
 return is own resolving map. The returned map has two top level keys:
 C<attributes> and C<operations>. Below these keys are the maps for attribute
@@ -321,11 +327,15 @@ return an arrayref in the form described above.
 
 =item *
 
-A coderef, which is executed when L<JMX::Jmx4Perl->get_attribute()> is called
-and which is supossed to do the complete lookup. This is the most flexible way
-for a handler to do anything he likes when an attribute value is requested or
+A coderef, which is executed when C<JMX::Jmx4Perl->get_attribute()> or
+C<JMX::Jmx4Perl->execute()> is called and which is supossed to do the complete
+lookup. The first argument to the subroutine is the handler which can be used
+to access the L<JMX::Jmx4Perl> object. The additional argument are either the
+value to set (for C<JMX::Jmx4Perl->set_attribute()> or the operation's
+arguments for C<JMX::Jmx4Perl->execute()>. This is the most flexible way for a
+handler to do anything it likes to do when an attribute value is requested or
 an operation is about to be executed. You have to return a
-L<JMX::Jmx4Perl::Response> object. 
+L<JMX::Jmx4Perl::Response> object.
 
 =back
 
@@ -405,7 +415,6 @@ specific information
 sub server_info { 
     my $self = shift;
     my $jmx4perl = $self->{jmx4perl};
-    
     my $ret = "";
     $ret .= sprintf("%-10.10s %s\n","Name:",$self->name);
     $ret .= sprintf("%-10.10s %s\n","Vendor:",$self->vendor) if $self->vendor;
@@ -492,6 +501,9 @@ sub jvm_info {
         my $sys_props = $self->_get_attribute(RUNTIME_SYSTEM_PROPERTIES);
         if ($sys_props) {
             $ret .= "System Properties:\n";
+            if (ref($sys_props) eq "HASH") {
+                $sys_props = [ values %$sys_props ];
+            }
             for my $prop (@{$sys_props}) {
                 $ret .= sprintf("   %-40.40s = %s\n",$prop->{key},$prop->{value});
             }
